package com.mh.common.utils;

import net.coobird.thumbnailator.Thumbnails;
import org.springframework.web.multipart.MultipartFile;
import sun.misc.BASE64Encoder;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * 文件相关操作
 * @author Louis
 * @date Jan 14, 2019
 */
public class FileUtils {

	/**
	 * 下载文件
	 * @param response
	 * @param file
	 * @param newFileName
	 */
	public static void downloadFile(HttpServletResponse response, File file, String newFileName) {
		try {
			response.setHeader("Content-Disposition", "attachment; filename=" + new String(newFileName.getBytes("ISO-8859-1"), "UTF-8"));
			BufferedOutputStream bos = new BufferedOutputStream(response.getOutputStream());
			InputStream is = new FileInputStream(file.getAbsolutePath());
			BufferedInputStream bis = new BufferedInputStream(is);
			int length = 0;
			byte[] temp = new byte[1 * 1024 * 10];
			while ((length = bis.read(temp)) != -1) {
				bos.write(temp, 0, length);
			}
			bos.flush();
			bis.close();
			bos.close();
			is.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static String convertFileToBase64(MultipartFile file) {
		byte[] data = null;
		// 读取文件字节数组
		try {
			InputStream in = file.getInputStream();
			data = new byte[in.available()];
			in.read(data);
			in.close();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		// 对字节数组进行编码，得到base64编码字符串
		BASE64Encoder encoder = new BASE64Encoder();
		return encoder.encode(data);
	}

	public static byte[] compressImage(MultipartFile file, long targetSize) throws IOException {
		try (InputStream inputStream = file.getInputStream()) {
			// 将输入流转换为字节数组
			byte[] originalBytes = new byte[inputStream.available()];
			inputStream.read(originalBytes);
			inputStream.close();
			ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(originalBytes);

			// 初始压缩比例
			double compressionRatio = 0.9;
			byte[] compressedBytes;

			while (true) {
				try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream()) {
					Thumbnails.of(byteArrayInputStream)
							.scale(compressionRatio)
							.outputQuality(compressionRatio)
							.toOutputStream(outputStream);

					compressedBytes = outputStream.toByteArray();

					if (compressedBytes.length <= targetSize) {
						return compressedBytes;
					} else {
						// 调整压缩比例
						compressionRatio -= 0.1;
						if (compressionRatio <= 0.1) {
							throw new RuntimeException("无法将图像压缩到目标大小");
						}
					}
				}

				// 重置输入流
				byteArrayInputStream.reset();
			}
		}
	}

	public static String reSizeImg(MultipartFile file) {
		try {
			byte[] bytes = compressImage(file, 512 * 1024);
			// 对字节数组进行编码，得到base64编码字符串
			BASE64Encoder encoder = new BASE64Encoder();
			return encoder.encode(bytes);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public static void copyFile(MultipartFile file, String uploadDir) {
		// 保存图片到本地
		try {
			// 确保目标目录存在
			File dir = new File(uploadDir);
			if (!dir.exists()) {
				dir.mkdirs();
			}
			// 文件保存路径
			Path path = Paths.get(uploadDir + file.getOriginalFilename());
			// 将上传文件保存到指定路径
			Files.copy(file.getInputStream(), path);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
