package com.mh.user.config.wechat;

import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.api.impl.WxMpServiceImpl;
import me.chanjar.weixin.mp.config.WxMpConfigStorage;
import me.chanjar.weixin.mp.config.impl.WxMpDefaultConfigImpl;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

import java.net.URLEncoder;

/**
 * @author chison
 * @date 2020-04-03 15:42
 * @Description 微信网页授权信息配置类
 */
@Component
@Configuration
public class WechatMpConfig {

    @Value("${wechat.mpAppId}")
    private String mpAppId;
    @Value("${wechat.mpAppSecret}")
    private String mpAppSecret;
    @Value("${wechat.redirectUri}")
    private String redirectUri;
    @Value("${wechat.authorizedUrl}")
    private String authorizedUrl;
    @Value("${wechat.access_token}")
    private String accessToken;
    @Value("${wechat.userinfo}")
    private String userinfo;
    @Value("${wechat.pushUrl}")
    private String pushUrl;
    @Value("${wechat.baseUrl}")
    private String baseUrl;

    public String getBaseUrl() {
        return baseUrl;
    }

    public void setBaseUrl(String baseUrl) {
        this.baseUrl = baseUrl;
    }

    public String getAuthorizedUrl() {
        return authorizedUrl.replace("mpAppId", mpAppId).replace("REDIRECT_URI", URLEncoder.encode(redirectUri));
    }

    public String getAccessTokenUrl() {
        return accessToken.replace("APPID", mpAppId).replace("SECRET", mpAppSecret);
    }

    public String getUserInfo(String accessToken, String openId) {
        return userinfo.replace("ACCESS_TOKEN", accessToken).replace("OPENID", openId);
    }

    public String getPushMsgUrl(String accessToken) {
        return pushUrl.replace("ACCESS_TOKEN", accessToken);
    }

    public String getMpAppId() {
        return mpAppId;
    }

    public String getMpAppSecret() {
        return mpAppSecret;
    }

    /**
     * 配置WxMpService所需信息
     *
     * @return
     */
    @Bean  // 此注解指定在Spring容器启动时，就执行该方法并将该方法返回的对象交由Spring容器管理
    public WxMpService wxMpService() {
        WxMpService wxMpService = new WxMpServiceImpl();
        // 设置配置信息的存储位置
        wxMpService.setWxMpConfigStorage(wxMpConfigStorage());

        return wxMpService;
    }

    public WxMpConfigStorage wxMpConfigStorage() {
        // 使用这个实现类则表示将配置信息存储在内存中
        WxMpDefaultConfigImpl wxMpInMemoryConfigStorage = new WxMpDefaultConfigImpl();
        wxMpInMemoryConfigStorage.setAppId(getMpAppId());
        wxMpInMemoryConfigStorage.setSecret(getMpAppSecret());

        return wxMpInMemoryConfigStorage;
    }
}
