package com.mh.user.controller;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.mh.common.http.HttpResult;
import com.mh.user.dto.DisplayBoxDataDTO;
import com.mh.user.dto.TableInfoDTO;
import com.mh.user.entity.*;
import com.mh.user.service.chillers.ChillersService;
import com.mh.user.service.chillers.DeviceDisplayService;
import com.mh.user.service.chillers.GaugeService;
import com.mh.user.vo.HostListVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author ljf
 * @title ：
 * @description ： 设备监控操作控制接口
 * @updateTime 2020-05-20
 * @throws ：
 */
@RestController
@RequestMapping("/chillers")
public class ChillersController {

    private final ChillersService chillersService;
    private final DeviceDisplayService deviceDisplayService;

    public ChillersController(ChillersService chillersService, DeviceDisplayService deviceDisplayService) {
        this.chillersService = chillersService;
        this.deviceDisplayService = deviceDisplayService;
    }

    /**
     * 查询主机列表详情信息
     * @param proId
     * @param systemId
     * @return
     */
    @GetMapping("/queryListByProId")
    public HttpResult queryChillersByProId(@RequestParam(value = "proId", required = false) String proId,
                                           @RequestParam("systemId") String systemId) {
        JSONArray resultJson = chillersService.queryChillersByProId(systemId, proId);
        return HttpResult.ok(resultJson);
    }

    /**
     * 查询主机详情主页信息：计算制冷量，用电量，碳排放，平均COP等
     * @param systemId
     * @param proId
     * @param deviceId
     * @return
     */
    @GetMapping("/queryHomeDetail/totalData")
    public HttpResult totalData(@RequestParam("systemId") String systemId,
                                      @RequestParam(value = "proId", required = false) String proId,
                                      @RequestParam("deviceId") String deviceId) {
        JSONObject resultJson = chillersService.queryTotalData(systemId, proId, deviceId);
        return HttpResult.ok(resultJson);
    }

    /**
     * 查询主机详情主页信息：当日能耗分析
     * @param systemId
     * @param proId
     * @param deviceId
     * @return
     */
    @GetMapping("/queryHomeDetail/chartData")
    public HttpResult chartData(@RequestParam("systemId") String systemId,
                                      @RequestParam(value = "proId", required = false) String proId,
                                      @RequestParam("deviceId") String deviceId) {
        JSONObject resultJson = chillersService.queryChartData(systemId, proId, deviceId);
        return HttpResult.ok(resultJson);
    }

}
