package com.mh.user.controller.wechat;

import com.alibaba.fastjson2.JSONObject;
import com.github.benmanes.caffeine.cache.Cache;
import com.mh.user.config.wechat.WechatMpConfig;
import com.mh.user.service.wechat.WechatUserInfoService;
import com.mh.user.utils.CacheUtil;
import com.mh.user.utils.SignUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.client.RestTemplate;

import java.util.concurrent.TimeUnit;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 微信相关接口
 * @date 2024-11-27 15:37:57
 */
@Slf4j
@Controller
@RequestMapping("/wechat")
public class WechatController {

    @Autowired
    private WechatMpConfig wechatMpConfig;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private WechatUserInfoService wechatUserInfoService;

    /**
     * 微信校验
     *
     * @param signature
     * @param timestamp
     * @param nonce
     * @param echostr
     * @return
     */
    @GetMapping("/checkToken")
    @ResponseBody
    public String checkSignature(@RequestParam("signature") String signature,
                                 @RequestParam("timestamp") String timestamp,
                                 @RequestParam("nonce") String nonce,
                                 @RequestParam("echostr") String echostr) {
        if (SignUtil.checkSignature(signature, timestamp, nonce)) {
            return echostr;
        } else {
            return null;
        }
    }

    @GetMapping("/authorize")
    @ResponseBody
    public String authorize() {
        log.info("进入authorize，redirect:{}", wechatMpConfig.getAuthorizedUrl());
        return "redirect:" + wechatMpConfig.getAuthorizedUrl();
    }

    @GetMapping("/userInfo")
    @ResponseBody
    public String userInfo(@RequestParam("code") String code, @RequestParam("state") String returnUrl) {
        try {
            // {"access_token":"86_OnLuaNu9da1Xeh3Q4PE77sLaX_CQQFNY5rvnWTK6o_-vhIeb74rydT93HCowiVKkYUAWPG8186mvQaP55Cy47Xtuo4EkqOSyiagyW1q4mVU","expires_in":7200,"refresh_token":"86_6YSDe0P0KkFnJH0KnJzvtodh9z62GdDbdl8aG-VF7Rn46DKsWvQOuES3EluhDUQ38uUboZNr9a5wU9LvoI9HDqhkWm9hoE-1T8vUPfjoLhc","openid":"o0qeN5lw4JquQy_H9Rk5QN-CyMRU","scope":"snsapi_userinfo"}
            // 判断cache中是否存在access_token
//            CacheUtil instance = CacheUtil.getInstance();
//            Cache<Object, Object> cache = instance.getCache("accessToken");
            String accessToken = "";
            String openId = "";
//            if (cache != null ) {
//                log.info("cache中存在access_token，直接使用");
//                accessToken = cache.getIfPresent("accessToken").toString();
//            } else {

                // 使用code换取access_token信息
                String getAccessTokenUrl = "https://api.weixin.qq.com/sns/oauth2/access_token?" +
                        "appid=" + wechatMpConfig.getMpAppId() + "&secret=" + wechatMpConfig.getMpAppSecret() + "&" +
                        "code=" + code + "&grant_type=authorization_code";
                String tokenResult = restTemplate.getForObject(getAccessTokenUrl, String.class);
                JSONObject jsonObj = JSONObject.parseObject(tokenResult);
                if (jsonObj == null || !jsonObj.containsKey("access_token")) {
                    return "获取access_token失败";
                }
                log.info("获取access_token==>{}", jsonObj.toJSONString());
                accessToken = jsonObj.get("access_token").toString();
                openId = jsonObj.get("openid").toString();
                // 存储access_token到缓存中
//                instance.createCache("accessToken", 6000, TimeUnit.SECONDS);
//            }

            String getUserInfoUrl = "https://api.weixin.qq.com/sns/userinfo?" +
                    "access_token=" + accessToken + "&openid=" + openId + "&lang=zh_CN";
            String userInfoResult = restTemplate.getForObject(getUserInfoUrl, String.class);
            userInfoResult = new String(userInfoResult.getBytes("ISO-8859-1"), "utf-8");
            System.out.println(userInfoResult);
            // 判断用户是否存在 0:不存在；1:存在
            int exist = wechatUserInfoService.existsUser(JSONObject.parseObject(userInfoResult));
            System.out.println("exist: " + exist);
            // 重定向到我们要跳转的页面
            String result = "";
            result = "redirect:" + wechatMpConfig.getBaseUrl() + "/#/?openId=" + openId;
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            return e.getMessage();
        }
    }

    @GetMapping("/getSub")
    @ResponseBody
    public void pushMsg(@RequestParam("openid") String openId,
                          @RequestParam("template_id") String templateId,
                          @RequestParam("action") String action,
                          @RequestParam("scene") String scene) {
        log.info("openId:{},template_id:{},action:{},scene:{}", openId, templateId, action, scene);
    }

}
