package com.mh.user.factory;

import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.MeterManageEntity;
import com.mh.user.strategy.ProtocolStrategy;
import lombok.extern.slf4j.Slf4j;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description modbus协议
 * @date 2024-09-12 16:38:44
 */
@Slf4j
public class ModbusProtocol implements Protocol {

    private ProtocolStrategy modbusProtocolStrategy;

    private static class SingletonHolder{
        private static final ModbusProtocol INSTANCE = new ModbusProtocol();
    }

    private ModbusProtocol(){
        // 防止外部直接实例化
    }

    public static ModbusProtocol getInstance(){
        return ModbusProtocol.SingletonHolder.INSTANCE;
    }

    @Override
    public void setStrategy(ProtocolStrategy protocolStrategy) {
        this.modbusProtocolStrategy = protocolStrategy;
    }

    @Override
    public String createOrder(MeterManageEntity meterManageEntity) {
        log.info("modbus标准协议：工厂创建报文");
        return modbusProtocolStrategy.createOrder(meterManageEntity);
    }

    @Override
    public String analysisReceiveData(DeviceCodeParamEntity deviceCodeParamEntity, String receiveData) {
        log.info("modbus标准协议：工厂解析报文");
        return modbusProtocolStrategy.analysisReceiveData(deviceCodeParamEntity, receiveData);
    }
}
