package com.mh.user.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.mh.user.entity.DeviceInstallEntity;
import com.mh.user.entity.DeviceInstallTempEntity;
import com.mh.user.mapper.provider.DeviceInstallProvider;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.mapping.StatementType;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Mapper
public interface DeviceInstallMapper extends BaseMapper<DeviceInstallEntity> {

    /**
     * 设备安装模块
     * 保存设备信息
     * @param deviceInstallEntity
     */
    @Insert("insert into device_code(device_addr,device_name,device_type,data_port,ratio,baud_rate,brand,project_id,install_position,install_date) values (" +
            " #{deviceAddr},#{deviceName},#{deviceType},#{dataCom},#{ratio},#{baudRate},#{brand},#{projectId},#{installPosition},getDate())")
    int saveDevice(DeviceInstallEntity deviceInstallEntity);

    /**
     * 设备安装模块
     * 保存设备信息
     * @param deviceInstallEntity
     */
    @Update("<script>" +
            "  update device_code set " +
            "  <if test='deviceName!=null'> device_name = #{deviceName} </if>" +
            "  <if test='deviceType!=null'> , device_type = #{deviceType} </if>" +
            "  <if test='dataCom!=null'> , data_port = #{dataPort} </if>" +
            "  <if test='ratio!=null'> , ratio = #{ratio} </if>" +
            "  <if test='baudRate!=null'> , baud_rate = #{baudRate} </if>" +
            "  <if test='brand!=null'> , brand = #{brand} </if>" +
            "  <if test='installPosition!=null'> , install_position = #{installPosition} </if>" +
            "  <if test='installer!=null'> , installer = #{installer} </if>" +
            "  <if test='projectId!=null'> , project_id = #{projectId} </if>" +
            "  <if test='remarks!=null'> , remarks = #{remarks} </if>" +
            "   where id = #{id} " +
            "</script>")
    int updateDevice(DeviceInstallEntity deviceInstallEntity);

    //按id更新水电表读数和采集日期
    @Update("update device_code  set last_value=#{lastValue},last_date=#{lastDate} where id=#{id}")
    void updateLastValue(@Param("id") Long id, @Param("lastValue") BigDecimal lastValue, @Param("lastDate") Date lastDate);
    /**
     * 设备管理模块：
     * 查询设备资料
     * @param page
     * @param limit
     * @return
     */
    @Results(id="rs", value = {
            @Result(property="deviceAddr",column="device_addr"),
            @Result(property="deviceName",column="deviceName"),
            @Result(property="deviceType",column="deviceType"),
            @Result(property="dataPort",column="dataPort"),
            @Result(property="standard",column="standard"),
            @Result(property ="ratio",column ="ratio"),
            @Result(property ="lastValue",column ="lastValue"),
            @Result(property ="lastDate",column ="lastDate"),
            @Result(property="isForbid",column="isForbid"),
            @Result(property="baudRate",column="baudrate"),
            @Result(property="parity",column="parity"),
            @Result(property="brand",column="brand"),
            @Result(property="installPosition",column="installPosition"),
            @Result(property="installDate",column="installDate"),
            @Result(property="remarks",column="remarks"),
            @Result(property ="projectId",column ="project_id"),
    })
    @Select("select * from ( " +
            "select *,ROW_NUMBER() over(order by id) as rn from device_code " +
            ") t where rn between (#{page}-1)*#{limit} and #{page}*#{limit}")
    List<DeviceInstallEntity> getAllDevice(@Param("page") int page, @Param("limit") int limit);

    //查询有多个采集参数的设备
    @Select("select  *  from  device_code  where  Project_id=#{ProjectID} ")
    List<DeviceInstallEntity> selectDeviceParams(@Param("ProjectID") String ProjectID);

    //查询有多个采集参数的设备
    @Select("select  *  from  device_code ")
    List<DeviceInstallEntity> selectDeviceParams2();

    //根据类型查询设备通讯号
    @Select("select  top 1 device_addr  from  device_code where  device_type=#{deviceType} and  project_id=#{projectId} ")
    String selectDeviceAddr(@Param("deviceType") String deviceType,@Param("projectId") String projectId);

    /**
     * 设备管理模块：
     * 根据条件获取设备查询的总条数
     * @return
     */
    @Select("select count(*) from  device_code where is_use=1 ")
    int getAllCount();


    @SelectProvider(type = DeviceInstallProvider.class,method = "queryDevice")
    List<DeviceInstallEntity> queryDevice(@Param("project_id") String projectId,
                                          @Param("deviceType") String deviceType,
                                          @Param("startDate") String startDate,
                                          @Param("endDate")String endDate,
                                          @Param("isUse")String isUse,
                                          @Param("isFault")String isFault,
                                          @Param("page") int page,
                                          @Param("limit") int limit);

    /**
     * 设备管理模块：
     * 根据条件获取设备查询的总条数
     * @return
     */
    @SelectProvider(type = DeviceInstallProvider.class,method = "getCount")
    int getCount(@Param("project_id") String projectId,
                 @Param("deviceType") String deviceType,
                 @Param("startDate") String startDate,
                 @Param("endDate")String endDate,
                 @Param("isUse")String isUse,
                 @Param("isFault")String isFault,
                 @Param("page") int page,
                 @Param("limit") int limit);

    //查询设备在线情况
    @SelectProvider(type = DeviceInstallProvider.class,method = "getIsOnlineCount")
    int getIsOnlineCount(@Param("isOnline") String isOnline,@Param("deviceType") String deviceType);

    //根据通讯地址和设备类型查询对应的设备信息
    @Select("select * from  device_code  where  device_addr=#{deviceAddr} and device_type=#{deviceType}  and project_id=#{projectId}")
    DeviceInstallEntity selectDevice(@Param("deviceAddr") String deviceAddr,@Param("deviceType") String deviceType,@Param("projectId") String projectId);

    /**
     * 设备管理模块：
     * 根据通讯地址删除设备信息
     * @param id
     */
    @Delete("delete from device_code where id=#{id}")
    int deleteDevice(String id);

    //EXCEL导入相关处理函数
    // 清临时表
    @Delete("DELETE FROM device_install_temp")
    void deleteDevice_install_temp();

    // 保存到临时表
    @Insert("insert into device_install_temp(device_addr,device_name,device_type,baudrate,data_com,ratio,building_id,row_id)" +
            " values(#{deviceAddr},#{deviceName},#{deviceType},#{baudRate},#{dataCom},#{ratio},#{buildingId},#{rowId})")
    void insertDevice_install_temp(@Param("deviceAddr")  String deviceAddr,
                                   @Param("deviceName")    String deviceName,
                                   @Param("deviceType")   String deviceType,
                                   @Param("baudRate")      int baudRate,
                                   @Param("dataCom")     String dataCom,
                                   @Param("ratio")     double ratio,
                                   @Param("buildingId")     String buildingId,
                                   @Param("rowId")          String rowId
    );

    // EXCEL导入查询EXCEL内容
    @Select("SELECT device_addr,device_name,device_type,baudRate,data_com,ratio, " +
            "       building_id,row_id,  " +
            "      (CASE " +
            "        when remarks is not null then remarks + ',行号为' + convert(varchar(100),Row_id) " +
            "        else '' " +
            "        end " +
            "       ) as remarks " +
            "   FROM device_install_temp " +
            "   ORDER BY ISNULL(remarks,'') DESC ,row_id  ")
    @Options(statementType = StatementType.CALLABLE)
    @Results({
            @Result(column = "device_addr",property = "deviceAddr" ),
            @Result(column = "device_name", property = "deviceName"),
            @Result(column = "device_type", property = "deviceType"),
            @Result(column = "baud_rate", property = "baudRate"),
            @Result(column = "data_com", property = "dataCom"),
            @Result(column = "ratio", property = "ratio"),
            @Result(column = "building_id", property = "buildingId"),
            @Result(column = "install_date", property = "installDate"),
            @Result(column = "remarks", property = "remarks"),
            @Result(column = "row_id", property = "rowId")
    })
    List<DeviceInstallTempEntity> queryExcelDevices();

    //查询Excel导入的数据的记录数
    @Select("SELECT COUNT(*) FROM device_install_temp ")
    int queryExcelDevicesCount();

    /**
     * @author nxr
     * @title ：判断导入资料数据的合法
     * @description ：在导入中的数据有重复
     * @updateTime 2022-06-19
     * @throws ：
     */
    @Update("update  device_install_temp  " +
            "  set Remarks = '数据中表号有重复'  " +
            "  where Remarks is null  " +
            "  and  device_addr in ( select  ISNULL(device_install_temp.device_addr,'')  " +
            "                            from  device_install_temp  " +
            "                          group by device_addr " +
            "                          having count(*) > 1)")
    void updateDevice_install_temp__multiple();

    //从临时表查询插入
    @Insert("insert into device_install(device_addr,device_name,device_type,data_com,baud_rate,ratio,building_id) select device_addr,device_name,device_type,data_com,baud_rate,ratio,building_id from device_install_temp")
    void insertFromDevice_install_temp();

    //修改启动状态
    @Update("update device_install set is_use=#{isUse}  where  device_addr=#{deviceAddr}")
    void updateDeviceIsUse(@Param("isUse") String isUse, @Param("deviceAddr")  String deviceAddr);

}
