package com.mh.user.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.mh.user.dto.EnergyMomYoyDataDTO;
import com.mh.user.entity.EnergyDataEntity;
import com.mh.user.mapper.provider.EnergyDataProvider;
import org.apache.ibatis.annotations.*;

import java.util.List;

@Mapper
public interface EnergyDataMapper extends BaseMapper<EnergyDataEntity> {

    //按10分钟
    @SelectProvider(type = EnergyDataProvider.class, method = "queryEnergyDataMi")
    @Results(id = "rs", value = {
            @Result(property = "curDate", column = "cur_date"),
            @Result(property = "projectId", column = "project_id"),
            @Result(property = "projectName", column = "project_name"),
            @Result(property = "ambTemp", column = "amb_Temp"),
            @Result(property = "humidity", column = "humidity"),
            @Result(property = "water", column = "water"),
            @Result(property = "elect", column = "elect"),
            @Result(property = "cl", column = "cl"),
            @Result(property = "cop", column = "cop")
    })
    List<EnergyDataEntity> queryEnergyDataMi(@Param("projectId") String projectId,
                                             @Param("curDate") String curDate,
                                             @Param("page") int page, @Param("limit") int limit);


    @SelectProvider(type = EnergyDataProvider.class, method = "getEnergyDataMiCount")
    int getEnergyDataMiCount(@Param("projectId") String projectId, @Param("curDate") String curDate);

    //按小时
    @SelectProvider(type = EnergyDataProvider.class, method = "queryEnergyDataHour")
    @Results({
            @Result(property = "id", column = "id"),
            @Result(property = "curDate", column = "cur_date"),
            @Result(property = "ambTemp", column = "amb_temp"),
            @Result(property = "humidity", column = "humidity"),
            @Result(property = "water", column = "water"),
            @Result(property = "elect", column = "elect"),
            @Result(property = "cl", column = "cl"),
            @Result(property = "cop", column = "cop"),
            @Result(property = "projectId", column = "project_id"),
            @Result(property = "projectName", column = "project_name")
    })
    List<EnergyDataEntity> queryEnergyDataHour(@Param("projectId") String projectId,
                                               @Param("curDate") String curDate,
                                               @Param("page") int page,
                                               @Param("limit") int limit);

    @SelectProvider(type = EnergyDataProvider.class, method = "getEnergyDataHourCount")
    int getEnergyDataHourCount(@Param("projectId") String projectId, @Param("curDate") String curDate);

    //按天
    @SelectProvider(type = EnergyDataProvider.class, method = "queryEnergyDataDay")
    @ResultMap("rs")
    List<EnergyDataEntity> queryEnergyDataDay(@Param("projectId") String projectId,
                                              @Param("curDate") String curDate,
                                              @Param("queryType") int queryType,
                                              @Param("page") int page,
                                              @Param("limit") int limit);

    @SelectProvider(type = EnergyDataProvider.class, method = "getEnergyDataDayCount")
    int getEnergyDataDayCount(@Param("projectId") String projectId, @Param("curDate") String curDate, @Param("queryType") int queryType);

    //按月
    @SelectProvider(type = EnergyDataProvider.class, method = "queryEnergyDataMonth")
    @ResultMap("rs")
    List<EnergyDataEntity> queryEnergyDataMonth(@Param("projectId") String projectId,
                                                @Param("curDate") String curDate,
                                                @Param("queryType") int queryTyp,
                                                @Param("page") int page, @Param("limit") int limit);

    @SelectProvider(type = EnergyDataProvider.class, method = "getEnergyDataMonthCount")
    int getEnergyDataMonthCount(@Param("projectId") String projectId, @Param("curDate") String curDate, @Param("queryType") int queryType);

    //按年
    @SelectProvider(type = EnergyDataProvider.class, method = "queryEnergyDataYear")
    @ResultMap("rs")
    List<EnergyDataEntity> queryEnergyDataYear(@Param("projectId") String projectId,
                                               @Param("startDate") String startDate,
                                               @Param("endDate") String endDate,
                                               @Param("queryType") int queryType,
                                               @Param("page") int page, @Param("limit") int limit);

    @SelectProvider(type = EnergyDataProvider.class, method = "getEnergyDataYearCount")
    int getEnergyDataYearCount(@Param("projectId") String projectId,
                               @Param("startDate") String startDate,
                               @Param("endDate") String endDate,
                               @Param("queryType") int queryType);

    @SelectProvider(type = EnergyDataProvider.class, method = "queryMomEnergyHourAndDayData")
    @Results(id = "mom", value = {
            @Result(property = "curDate", column = "cur_date"),
            @Result(property = "projectId", column = "project_id"),
            @Result(property = "projectName", column = "project_name"),
            @Result(property = "ambTemp", column = "amb_Temp"),
            @Result(property = "humidity", column = "humidity"),
            @Result(property = "elect", column = "elect"),
            @Result(property = "cl", column = "cl"),
            @Result(property = "cop", column = "cop"),
            @Result(property = "last_elect", column = "lastElect"),
            @Result(property = "last_cl", column = "lastCl"),
            @Result(property = "last_cop", column = "lastCop"),
            @Result(property = "elect_ratio", column = "electRatio"),
            @Result(property = "cl_ratio", column = "clRatio"),
            @Result(property = "cop_ratio", column = "copRatio")
    })
    List<EnergyMomYoyDataDTO> queryMomEnergyHourAndDayData(@Param("tableName") String tableName,
                                                           @Param("projectId") String projectId,
                                                           @Param("startDate") String startDate,
                                                           @Param("endDate") String endDate,
                                                           @Param("lastStartDate") String lastStartDate,
                                                           @Param("lastEndDate") String lastEndDate,
                                                           @Param("dateType") String dateType);

    @SelectProvider(type = EnergyDataProvider.class, method = "queryYoyEnergyHourAndDayData")
    @Results(id = "yoy", value = {
            @Result(property = "curDate", column = "cur_date"),
            @Result(property = "projectId", column = "project_id"),
            @Result(property = "projectName", column = "project_name"),
            @Result(property = "ambTemp", column = "amb_Temp"),
            @Result(property = "humidity", column = "humidity"),
            @Result(property = "elect", column = "elect"),
            @Result(property = "cl", column = "cl"),
            @Result(property = "cop", column = "cop"),
            @Result(property = "last_elect", column = "lastElect"),
            @Result(property = "last_cl", column = "lastCl"),
            @Result(property = "last_cop", column = "lastCop"),
            @Result(property = "elect_ratio", column = "electRatio"),
            @Result(property = "cl_ratio", column = "clRatio"),
            @Result(property = "cop_ratio", column = "copRatio")
    })
    List<EnergyMomYoyDataDTO> queryYoyEnergyHourAndDayData(@Param("tableName") String tableName,
                                                           @Param("projectId") String projectId,
                                                           @Param("startDate") String startDate,
                                                           @Param("endDate") String endDate,
                                                           @Param("lastStartDate") String lastStartDate,
                                                           @Param("lastEndDate") String lastEndDate,
                                                           @Param("dateType") String dateType);

    @Select("select top 1 edd.*,pi.project_name from energy_data_day edd " +
            " left join project_info pi on edd.project_id = pi.id " +
            " where cur_date = #{curDate} and project_id = #{projectId} ")
    @Results({
            @Result(property = "id", column = "id"),
            @Result(property = "curDate", column = "cur_date"),
            @Result(property = "ambTemp", column = "amb_temp"),
            @Result(property = "humidity", column = "humidity"),
            @Result(property = "water", column = "water"),
            @Result(property = "elect", column = "elect"),
            @Result(property = "cl", column = "cl"),
            @Result(property = "cop", column = "cop"),
            @Result(property = "projectId", column = "project_id"),
            @Result(property = "projectName", column = "project_name")
    })
    EnergyDataEntity queryEnergyDataByDate(@Param("curDate") String curDate,
                                           @Param("projectId") String projectId);

    @Select("select edd.*,pi.project_name from energy_data_day edd " +
            " left join project_info pi on edd.project_id = pi.id " +
            " where cur_date >= #{startDate} and cur_date <= #{curDate} and project_id = #{projectId} ")
    @Results({
            @Result(property = "id", column = "id"),
            @Result(property = "curDate", column = "cur_date"),
            @Result(property = "ambTemp", column = "amb_temp"),
            @Result(property = "humidity", column = "humidity"),
            @Result(property = "water", column = "water"),
            @Result(property = "elect", column = "elect"),
            @Result(property = "cl", column = "cl"),
            @Result(property = "cop", column = "cop"),
            @Result(property = "projectId", column = "project_id"),
            @Result(property = "projectName", column = "project_name")
    })
    List<EnergyDataEntity> queryEnergyDataByDateRange(@Param("startDate") String startDate,
                                                @Param("curDate") String curDate,
                                                @Param("projectId") String projectId);
}
