package com.mh.user.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.github.benmanes.caffeine.cache.Cache;
import com.mh.user.entity.*;
import com.mh.user.factory.Protocol;
import com.mh.user.factory.ProtocolFactory;
import com.mh.user.mapper.DeviceCodeParamMapper;
import com.mh.user.service.*;
import com.mh.user.strategy.ProtocolStrategy;
import com.mh.user.strategy.ProtocolStrategyFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

@Transactional(rollbackFor = Exception.class)
@Service
@Slf4j
public class DeviceCodeParamServiceImpl implements DeviceCodeParamService {

    @Autowired
    DeviceCodeParamMapper deviceCodeParamMapper;

    @Autowired
    private ProjectInfoService projectInfoService;

    @Autowired
    private MeterManageService meterManageService;

    @Autowired
    private GatewayManageService gatewayManageService;

    @Autowired
    private DeviceParamsService deviceParamsService;

    @Autowired
    @Qualifier("caffeineCache")
    private Cache caffeineCache;


    @Override
    public void insertDeviceCodeParam(DeviceCodeParamEntity deviceCodeParamEntity) {
        deviceCodeParamMapper.insertDeviceCodeParam(deviceCodeParamEntity);
    }

    @Override
    public List<DeviceCodeParamEntity> queryCodeParam(String gatewayPort) {
        return deviceCodeParamMapper.queryCodeParam(gatewayPort);
    }

    @Override
    public List<DeviceCodeParamEntity> queryCodeParam2(String comName) {
        return deviceCodeParamMapper.queryCodeParam2(comName);
    }

    @Override
    public List<DeviceCodeParamEntity> queryCodeParam3() {
        return deviceCodeParamMapper.queryCodeParam3();
    }

    @Override
    public DeviceCodeParamEntity queryDeviceCodeParam(String projectId, String deviceType, String deviceAddr,String registerAddr) {
        return deviceCodeParamMapper.queryDeviceCodeParam(projectId,deviceType,deviceAddr,registerAddr);
    }

    @Override
    public int queryCount(String ProjectID) {
        return deviceCodeParamMapper.queryCount(ProjectID);
    }

    @Override
    public int queryCount2() {
        return deviceCodeParamMapper.queryCount2();
    }

    @Override
    public void insertDeviceCodeParamList(List<DeviceCodeParamEntity> deviceCodeParamEntityList) {
        deviceCodeParamMapper.insertDeviceCodeParamList(deviceCodeParamEntityList);
    }

    @Override
    public void selectInsertDeviceCodeParam(String ProjectID) {
        deviceCodeParamMapper.selectInsertDeviceCodeParam(ProjectID);
    }

    @Override
    public void createCodeParams(MeterManageEntity entity) {
        if (entity == null) {
            List<ProjectInfoEntity> projectInfoEntities = projectInfoService.queryProjectInfo("1", "");
            if (null != projectInfoEntities && !projectInfoEntities.isEmpty()) {
                for (ProjectInfoEntity projectInfoEntity : projectInfoEntities) {
                    // getReadOrder485.createOrderParam(String.valueOf(projectInfoEntity.getId()));    //生成采集参数,1新珠江（6000）,2广合科技（6001）,3广大科技（6002）,6珠江国际,7保利山庄,8东莞迎宾馆
                    createOrderParamByStrategy(projectInfoEntity.getSystemId(), String.valueOf(projectInfoEntity.getId()));
                }
            }
            return;
        }
        Long projectId = entity.getProjectId();
        if (projectId != 0) {
            // 删除数据库存储的采集参数
            deviceCodeParamMapper.deleteDeviceCodeParam(projectId);
            // 删除缓存
            caffeineCache.invalidate("code_params_"+projectId);
            List<DeviceCodeParamEntity> deviceCodeParamEntityList = createOrderParamByStrategy("1", String.valueOf(projectId));
            // 放入缓存中
            if (deviceCodeParamEntityList != null) {
                caffeineCache.put("code_params_" + projectId, deviceCodeParamEntityList);
            }
            List<DeviceCodeParamEntity> test = (List<DeviceCodeParamEntity>) caffeineCache.getIfPresent("code_params_"+projectId);
            log.info("生成采集参数完毕==>{}", JSONObject.toJSONString(test));
        }
    }

    public List<DeviceCodeParamEntity> createOrderParamByStrategy(String systemId, String projectId) {
        int r;
        if (projectId != null && !projectId.isEmpty()) {
            r = queryCount(projectId);//查询记录数
        } else {
            r = queryCount2();//查询记录数
        }
        if (r > 0) {
            return null;
        }
        List<DeviceCodeParamEntity> deviceCodeParamEntityList = new ArrayList<>();
        List<MeterManageEntity> meterManageEntities = Collections.emptyList();

        if (projectId != null && !projectId.isEmpty()) {
            meterManageEntities = meterManageService.queryBySystemIdAndProjectId(systemId, projectId);
        }

        DeviceCodeParamEntity deviceCodeParamEntity = new DeviceCodeParamEntity();

        // 根据协议进行匹配生成采集报文
        for (MeterManageEntity meterManageEntity : meterManageEntities) {
            // 重置采集参数
            deviceCodeParamEntity.reset();

            // 创建设备报文
            String protocolType = String.valueOf(meterManageEntity.getProtocolType());
            Protocol protocol = ProtocolFactory.matchProtocol(protocolType);
            ProtocolStrategy strategy = ProtocolStrategyFactory.matchProtocolStrategy(protocolType);

            if (strategy == null) {
                continue;
            }

            protocol.setStrategy(strategy);
            String sendStr = protocol.createOrder(meterManageEntity);
            log.info("采集指令===> {}", sendStr);

            deviceCodeParamEntity.setDeviceAddr(meterManageEntity.getMtCode());
            deviceCodeParamEntity.setDeviceName(meterManageEntity.getMtName());
            deviceCodeParamEntity.setDeviceType(String.valueOf(meterManageEntity.getMtType()));
            deviceCodeParamEntity.setStrData(sendStr);

            try {
                GatewayManageEntity gatewayManageEntity = gatewayManageService.findById(meterManageEntity.getGatewayId());
                if (gatewayManageEntity != null) {
                    deviceCodeParamEntity.setDataPort(String.valueOf(gatewayManageEntity.getPort()));
                }
            } catch (Exception e) {
                log.error("查询网关信息失败: {}", e.getMessage());
            }

            try {
                DeviceParamsEntity paramsEntity = deviceParamsService.findById(meterManageEntity.getParamId());
                if (paramsEntity != null) {
                    deviceCodeParamEntity.setBaudRate(paramsEntity.getBaudRate());
                    deviceCodeParamEntity.setParity(paramsEntity.getParity());
                }
            } catch (Exception e) {
                log.error("查询设备参数失败: {}", e.getMessage());
            }

            deviceCodeParamEntity.setBrand(meterManageEntity.getMtBrand());
            deviceCodeParamEntity.setFunCode(meterManageEntity.getFuncCode());
            deviceCodeParamEntity.setRegisterAddr(meterManageEntity.getRegisterAddr());
            deviceCodeParamEntity.setRegisterName(meterManageEntity.getOtherName());
            deviceCodeParamEntity.setDigit(meterManageEntity.getDigits());
            deviceCodeParamEntity.setGrade(meterManageEntity.getGrade());
            deviceCodeParamEntity.setDataValue(null); // 传入值(目前不需要)
            deviceCodeParamEntity.setProjectId(projectId);
            deviceCodeParamEntity.setCreateTime(new Date());

            deviceCodeParamEntity.setMmId(meterManageEntity.getId());
            deviceCodeParamEntity.setDataType(meterManageEntity.getDataType());
            deviceCodeParamEntity.setProtocolType(meterManageEntity.getProtocolType());

            deviceCodeParamEntityList.add(deviceCodeParamEntity.clone());
        }

        if (!deviceCodeParamEntityList.isEmpty()) {
            insertDeviceCodeParamList(deviceCodeParamEntityList);
        }

        return deviceCodeParamEntityList;

    }
}
