package com.mh.user.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.mh.common.page.MybatisPageHelper;
import com.mh.common.page.PageRequest;
import com.mh.common.page.PageResult;
import com.mh.common.utils.StringUtils;
import com.mh.user.dto.EnergyMomYoyDataDTO;
import com.mh.user.entity.DataResultChEntity;
import com.mh.user.entity.EnergyDataEntity;
import com.mh.user.mapper.EnergyDataMapper;
import com.mh.user.model.EnergyModel;
import com.mh.user.service.EnergyDataService;
import com.mh.user.utils.DateUtil;
import com.mh.user.utils.ExchangeStringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.YearMonth;
import java.time.format.DateTimeFormatter;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
public class EnergyDataServiceImpl implements EnergyDataService {

    @Autowired
    EnergyDataMapper energyDataMapper;

    @Override
    public List<EnergyDataEntity> queryEnergyData(EnergyModel energyModel) {
        String projectId = energyModel.getProjectId();
        String startDate = energyModel.getStartDate();
        String endDate = energyModel.getEndDate();
        int page = energyModel.getPage();
        int limit = energyModel.getLimit();
        int type = energyModel.getType();
        int queryType = energyModel.getQueryType();

        if (type == 1) { //按十分钟
            return energyDataMapper.queryEnergyDataMi(projectId, startDate, page, limit);
        } else if (type == 2) { //按小时
            // 如果项目是20，则查询两个projectId 14和15
            if (projectId.equals("20")) {
                List<EnergyDataEntity> data1 = energyDataMapper.queryEnergyDataHour("14", startDate, page, limit);
                List<EnergyDataEntity> data2 = energyDataMapper.queryEnergyDataHour("15", startDate, page, limit);
                return getEnergyDataEntities(data1, data2);
            }
            return energyDataMapper.queryEnergyDataHour(projectId, startDate, page, limit);
        } else if (type == 3) { //按天
            if (projectId.equals("20")) {
                List<EnergyDataEntity> data1 = energyDataMapper.queryEnergyDataDay("14", startDate, queryType, page, limit);
                List<EnergyDataEntity> data2 = energyDataMapper.queryEnergyDataDay("15", startDate, queryType, page, limit);
                // 两个列表根据时间分组合并数据
                // 合并两个列表并根据时间分组
                return getEnergyDataEntities(data1, data2);
            }
            return energyDataMapper.queryEnergyDataDay(projectId, startDate, queryType, page, limit);
        } else if (type == 4) { //按月
            if (projectId.equals("20")) {
                List<EnergyDataEntity> data1 = energyDataMapper.queryEnergyDataMonth("14", startDate, queryType, page, limit);
                List<EnergyDataEntity> data2 = energyDataMapper.queryEnergyDataMonth("15", startDate, queryType, page, limit);
                // 两个列表根据时间分组合并数据
                // 合并两个列表并根据时间分组
                return getEnergyDataEntities(data1, data2);
            }
            return energyDataMapper.queryEnergyDataMonth(projectId, startDate, queryType, page, limit);
        } else if (type == 5) { //按年
            if (projectId.equals("20")) {
                List<EnergyDataEntity> data1 = energyDataMapper.queryEnergyDataYear("14", startDate, endDate, queryType, page, limit);
                List<EnergyDataEntity> data2 = energyDataMapper.queryEnergyDataYear("15", startDate, endDate, queryType, page, limit);
                // 两个列表根据时间分组合并数据
                // 合并两个列表并根据时间分组
                return getEnergyDataEntities(data1, data2);
            }
            return energyDataMapper.queryEnergyDataYear(projectId, startDate, endDate, queryType, page, limit);
        } else {  //按天，默认
            return energyDataMapper.queryEnergyDataDay(projectId, startDate, queryType, page, limit);
        }
    }

    private static List<EnergyDataEntity> getEnergyDataEntities(List<EnergyDataEntity> data1, List<EnergyDataEntity> data2) {
        // 两个列表根据时间分组合并数据
        // 合并两个列表并根据时间分组
        Map<String, EnergyDataEntity> mergedMap = Stream.concat(data1.stream(), data2.stream())
                .collect(Collectors.toMap(
                        entity -> entity.getCurDate(),
                        entity -> entity,
                        (entity1, entity2) -> {
                            entity1.setCl(BigDecimal.valueOf(entity1.getCl()).add(BigDecimal.valueOf(entity2.getCl())
                                    .setScale(2, RoundingMode.HALF_UP)).doubleValue());
                            return entity1;
                        }
                ));

        // 重新赋值项目名称并创建新的列表
        List<EnergyDataEntity> mergedList = mergedMap.values().stream()
                .map(entity -> {
                    entity.setProjectName("河北珠江啤酒"); // 根据需要重新赋值项目名称
                    entity.setProjectId("20");
                    return entity;
                })
                .collect(Collectors.toList());

        // 按照时间字符“yyyy-MM-dd HH”进行排序
        mergedList.sort(Comparator.comparing(EnergyDataEntity::getCurDate));

        return mergedList;
    }

    @Override
    public int getEnergyDataCount(EnergyModel energyModel) {
        String projectId = energyModel.getProjectId();
        String startDate = energyModel.getStartDate();
        String endDate = energyModel.getEndDate();
        int type = energyModel.getType();
        int queryType = energyModel.getQueryType();

        if (type == 1) { //按十分钟
            return energyDataMapper.getEnergyDataMiCount(projectId, startDate);
        } else if (type == 2) { //按小时
            return energyDataMapper.getEnergyDataHourCount(projectId, startDate);
        } else if (type == 3) { //按天
            return energyDataMapper.getEnergyDataDayCount(projectId, startDate, queryType);
        } else if (type == 4) { //按月
            return energyDataMapper.getEnergyDataMonthCount(projectId, startDate, queryType);
        } else if (type == 5) { //按年
            return energyDataMapper.getEnergyDataYearCount(projectId, startDate, endDate, queryType);
        } else {  //按天，默认
            return energyDataMapper.getEnergyDataDayCount(projectId, startDate, queryType);
        }
    }

    @Override
    public PageResult queryMomEnergyData(EnergyModel energyModel) {
        String projectId = energyModel.getProjectId();
        String startDate = energyModel.getStartDate();
        String endDate = energyModel.getEndDate();
        int page = energyModel.getPage();
        int limit = energyModel.getLimit();
        int type = energyModel.getType();
        int queryType = energyModel.getQueryType();

        String lastStartDate = "";
        String lastEndDate = "";
        String tableName = "energy_data_hour";
        String dateType = "hour";

        if (StringUtils.isBlank(endDate)) {
            endDate = startDate;
        }

        switch (type) {
            case 1:
                // 按十分钟查询
                break;
            case 2:
                // 按小时查询
                startDate = startDate + " 00";
                endDate = endDate + " 23";

                // 定义日期时间格式
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH");

                // startDate 减一个小时
                LocalDateTime startDateTime = LocalDateTime.parse(startDate, formatter);
                LocalDateTime newStartDateTime = startDateTime.minusHours(1);
                lastStartDate = newStartDateTime.format(formatter);


                // endDate 减一个小时
                LocalDateTime endDateTime = LocalDateTime.parse(endDate, formatter);
                LocalDateTime newEndDateTime = endDateTime.minusHours(1);
                lastEndDate = newEndDateTime.format(formatter);
                break;
            case 3:
                tableName = "energy_data_day";
                dateType = "day";
                // 按天查询
                YearMonth yearMonthStart = YearMonth.parse(startDate, DateTimeFormatter.ofPattern("yyyy-MM"));
                LocalDate firstDayOfMonth = yearMonthStart.atDay(1);
                LocalDate lastDayOfMonth = yearMonthStart.atEndOfMonth();

                startDate = firstDayOfMonth.toString();
                endDate = lastDayOfMonth.toString();

                // 定义日期时间格式
                DateTimeFormatter dayMatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

                // startDate 减一天
                LocalDateTime startDayDate = LocalDateTime.parse(startDate + "T00:00:00", DateTimeFormatter.ISO_LOCAL_DATE_TIME);
                LocalDateTime newStartDayDate = startDayDate.minusDays(1);
                lastStartDate = newStartDayDate.format(dayMatter);

                // endDate 减一天
                LocalDateTime endDayDate = LocalDateTime.parse(endDate + "T23:59:59", DateTimeFormatter.ISO_LOCAL_DATE_TIME);
                LocalDateTime newEndDayDate = endDayDate.minusDays(1);
                if (newEndDayDate.getDayOfMonth() == 1) {
                    newEndDayDate = newEndDayDate.minusDays(1).withHour(23).withMinute(59).withSecond(59);
                }
                lastEndDate = newEndDayDate.format(dayMatter);
                break;
            case 4:
                tableName = "energy_data_month";
                dateType = "month";
                // 假设传入的时间格式为 "2023"
                String yearStr = startDate;

                // 1. 获取当前年份
                int year = Integer.parseInt(yearStr);

                // 2. 计算今年的开始月份
                int startMonth = 1;

                // 3. 获取当前月份
                LocalDate now = LocalDate.now();
                int currentMonth = now.getMonthValue();

                // 4. 计算环比月份
                int yearForComparison = year;
                int monthForComparison = currentMonth - 1;
                if (monthForComparison == 0) {
                    monthForComparison = 12;
                    yearForComparison = year - 1;
                }

                int lastMonth = startMonth - 1;
                int lastYear = year;
                if (lastMonth == 0) {
                    lastMonth = 12;
                    lastYear = year - 1;
                }

                // 当前时间
                startDate = year + "-" + ExchangeStringUtil.addZeroForNum(String.valueOf(startMonth), 2);
                endDate = year + "-" + ExchangeStringUtil.addZeroForNum(String.valueOf(currentMonth), 2);

                // 环比时间
                lastStartDate = lastYear + "-" + ExchangeStringUtil.addZeroForNum(String.valueOf(lastMonth), 2);
                lastEndDate = yearForComparison + "-" + ExchangeStringUtil.addZeroForNum(String.valueOf(monthForComparison), 2);
                break;
            case 5:
                tableName = "energy_data_year";
                dateType = "year";
                // 假设传入的时间格式为 "2023"
                // 1. 获取当前年份
                int startYear = Integer.parseInt(startDate);
                int endYear = Integer.parseInt(endDate);

                // 环比时间
                lastStartDate = String.valueOf(startYear - 1);
                lastEndDate = String.valueOf(endYear - 1);
                break;
            default:
                break;
        }
        PageRequest pageRequest = new PageRequest();
        if (page != 0) {
            PageHelper.startPage(page, limit);
            pageRequest.setPageNum(page);
            pageRequest.setPageSize(limit);
        }
        List<EnergyMomYoyDataDTO> resultList = energyDataMapper.queryMomEnergyHourAndDayData(tableName,
                projectId,
                startDate,
                endDate,
                lastStartDate,
                lastEndDate,
                dateType);
        return MybatisPageHelper.getPageResult(pageRequest, new PageInfo<>(resultList));
    }

    @Override
    public PageResult queryYoyEnergyData(EnergyModel energyModel) {
        String projectId = energyModel.getProjectId();
        String startDate = energyModel.getStartDate();
        String endDate = energyModel.getEndDate();
        int page = energyModel.getPage();
        int limit = energyModel.getLimit();
        int type = energyModel.getType();
        int queryType = energyModel.getQueryType();

        String lastStartDate = "";
        String lastEndDate = "";
        String tableName = "energy_data_hour";
        String dateType = "hour";

        if (StringUtils.isBlank(endDate)) {
            endDate = startDate;
        }

        switch (type) {
            case 1:
                // 按十分钟查询
                break;
            case 2:
                // 按小时查询
                startDate = startDate + " 00";
                endDate = endDate + " 23";

                // 定义日期时间格式
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH");

                // startDate 减一年
                LocalDateTime startDateTime = LocalDateTime.parse(startDate, formatter);
                LocalDateTime newStartDateTime = startDateTime.minusYears(1);
                lastStartDate = newStartDateTime.format(formatter);


                // endDate 减一年
                LocalDateTime endDateTime = LocalDateTime.parse(endDate, formatter);
                LocalDateTime newEndDateTime = endDateTime.minusYears(1);
                lastEndDate = newEndDateTime.format(formatter);
                break;
            case 3:
                tableName = "energy_data_day";
                dateType = "day";
                // 按天查询
                YearMonth yearMonthStart = YearMonth.parse(startDate, DateTimeFormatter.ofPattern("yyyy-MM"));
                LocalDate firstDayOfMonth = yearMonthStart.atDay(1);
                LocalDate lastDayOfMonth = yearMonthStart.atEndOfMonth();

                startDate = firstDayOfMonth.toString();
                endDate = lastDayOfMonth.toString();

                // 定义日期时间格式
                DateTimeFormatter dayMatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

                // startDate 减一年
                LocalDateTime startDayDate = LocalDateTime.parse(startDate + "T00:00:00", DateTimeFormatter.ISO_LOCAL_DATE_TIME);
                LocalDateTime newStartDayDate = startDayDate.minusYears(1);
                lastStartDate = newStartDayDate.format(dayMatter);

                // endDate 减一年
                LocalDateTime endDayDate = LocalDateTime.parse(endDate + "T23:59:59", DateTimeFormatter.ISO_LOCAL_DATE_TIME);
                LocalDateTime newEndDayDate = endDayDate.minusYears(1);
                if (newEndDayDate.getDayOfMonth() == 1) {
                    newEndDayDate = newEndDayDate.minusYears(1).withHour(23).withMinute(59).withSecond(59);
                }
                lastEndDate = newEndDayDate.format(dayMatter);
                break;
            case 4:
                tableName = "energy_data_month";
                dateType = "month";
                // 假设传入的时间格式为 "2023"
                String yearStr = startDate;

                // 1. 获取当前年份
                int year = Integer.parseInt(yearStr);

                // 2. 计算今年的开始月份
                int startMonth = 1;

                // 3. 获取当前月份
                LocalDate now = LocalDate.now();
                int currentMonth = now.getMonthValue();

                // 当前时间
                startDate = year + "-" + ExchangeStringUtil.addZeroForNum(String.valueOf(startMonth), 2);
                endDate = year + "-" + ExchangeStringUtil.addZeroForNum(String.valueOf(currentMonth), 2);

                // 环比时间
                lastStartDate = year - 1 + "-" + ExchangeStringUtil.addZeroForNum(String.valueOf(startMonth), 2);
                lastEndDate = year - 1 + "-" + ExchangeStringUtil.addZeroForNum(String.valueOf(currentMonth), 2);
                break;
            case 5:
                tableName = "energy_data_year";
                dateType = "year";
                // 假设传入的时间格式为 "2023"
                // 1. 获取当前年份
                int startYear = Integer.parseInt(startDate);
                int endYear = Integer.parseInt(endDate);

                // 环比时间
                lastStartDate = String.valueOf(startYear - 1);
                lastEndDate = String.valueOf(endYear - 1);
                break;
            default:
                break;
        }
        PageRequest pageRequest = new PageRequest();
        if (page != 0) {
            PageHelper.startPage(page, limit);
            pageRequest.setPageNum(page);
            pageRequest.setPageSize(limit);
        }
        List<EnergyMomYoyDataDTO> resultList = energyDataMapper.queryYoyEnergyHourAndDayData(tableName,
                projectId,
                startDate,
                endDate,
                lastStartDate,
                lastEndDate,
                dateType);
        return MybatisPageHelper.getPageResult(pageRequest, new PageInfo<>(resultList));
    }

    @Override
    public EnergyDataEntity queryEnergyDataByDate(String curDate, String projectId) {
        return energyDataMapper.queryEnergyDataByDate(curDate, projectId);
    }

    @Override
    public List<EnergyDataEntity> queryEnergyDataByDateRange(String startDate, String curDate, String projectId) {
        return energyDataMapper.queryEnergyDataByDateRange(startDate, curDate, projectId);
    }
}
