package com.mh.user.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.mh.common.page.MybatisPageHelper;
import com.mh.common.page.PageRequest;
import com.mh.common.page.PageResult;
import com.mh.common.utils.FileUtils;
import com.mh.common.utils.StringUtils;
import com.mh.user.constants.Constant;
import com.mh.user.entity.ProjectInfoEntity;
import com.mh.user.mapper.ProjectInfoMapper;
import com.mh.user.service.ProjectInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Service
public class ProjectInfoServiceImpl implements ProjectInfoService {

    @Autowired
    private ProjectInfoMapper projectInfoMapper;


    @Override
    public List<ProjectInfoEntity> queryProjectInfo(String systemID, String account) {

        return projectInfoMapper.queryProjectInfo(systemID, account);
    }

    @Override
    public String selectName(String id) {
        return projectInfoMapper.selectName(id);
    }

    @Override
    public PageResult queryByPage(PageRequest pageRequest) {
        String systemID = StringUtils.getColumnFilterValue(pageRequest, "systemId");
        String projectId = StringUtils.getColumnFilterValue(pageRequest, "projectId");
        // 设置分页参数
        int pageNum = pageRequest.getPageNum();
        int pageSize = pageRequest.getPageSize();
        if (pageNum != 0) {
            PageHelper.startPage(pageNum, pageSize);
        }
        List<ProjectInfoEntity> result = projectInfoMapper.queryProjectInfo(systemID, projectId);
        return MybatisPageHelper.getPageResult(pageRequest, new PageInfo<>(result));
    }

    @Override
    public void update(ProjectInfoEntity projectInfoEntity) {
        if (null != projectInfoEntity.getFile()) {
            String currentPath = System.getProperty("user.dir");
            FileUtils.copyFile(projectInfoEntity.getFile(), currentPath + Constant.PROJECT_IMG_PATH);
            projectInfoEntity.setPicContent("/images/"+projectInfoEntity.getFile().getOriginalFilename());
            projectInfoEntity.setPic(projectInfoEntity.getFile().getOriginalFilename());
        }
        projectInfoMapper.updateById(projectInfoEntity);
    }

    @Override
    public void save(ProjectInfoEntity projectInfoEntity) {
        if (null != projectInfoEntity.getFile()) {
//            String fileToBase64 = FileUtils.reSizeImg(projectInfoEntity.getFile());
//            projectInfoEntity.setPicContent(fileToBase64);
//            projectInfoEntity.setPic(projectInfoEntity.getFile().getOriginalFilename());
            String currentPath = System.getProperty("user.dir");
            FileUtils.copyFile(projectInfoEntity.getFile(), currentPath + Constant.PROJECT_IMG_PATH);
            projectInfoEntity.setPicContent("/images/"+projectInfoEntity.getFile().getOriginalFilename());
            projectInfoEntity.setPic(projectInfoEntity.getFile().getOriginalFilename());
        }
        projectInfoEntity.setCreateTime(new Date());
        projectInfoMapper.insert(projectInfoEntity);
    }

    @Override
    public void delete(String id) {
        projectInfoMapper.deleteById(id);
    }

    @Override
    public ProjectInfoEntity findById(Long id) {
        return projectInfoMapper.selectById(id);
    }
}
