package com.mh.user.service.impl;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.mh.user.dto.ScreenRequestParamDTO;
import com.mh.user.mapper.ScreenMapper;
import com.mh.user.service.ScreenService;
import com.mh.user.utils.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 大屏服务实现类
 * @date 2024-09-24 14:52:28
 */
@Slf4j
@Service
public class ScreenServiceImpl implements ScreenService {

    @Resource
    private ScreenMapper screenMapper;

    @Override
    public JSONArray faultList() {
        JSONArray resultJson = new JSONArray();
        List<Map<String, Object>> faultList = screenMapper.queryFaultList();
        if (faultList.isEmpty()) {
            return resultJson;
        }
        // 根据项目名称分组
        Map<Object, List<Map<String, Object>>> projectIds = faultList.stream().collect(Collectors.groupingBy(map -> map.get("project_name")));
        for (Map.Entry<Object, List<Map<String, Object>>> entry : projectIds.entrySet()) {
            String projectName = (String) entry.getKey();
            List<Map<String, Object>> projectId = faultList.stream().filter(map -> map.get("project_name").equals(projectName)).collect(Collectors.toList());
            JSONObject data = new JSONObject();
            data.put("projectName", projectName);
            data.put("faultNum", projectId.size());
            String[] faultListStr = new String[projectId.size()];
            for (int i = 0; i < projectId.size(); i++) {
                faultListStr[i] = (String) projectId.get(i).get("other_name");
            }
            data.put("faultList", faultListStr);
            resultJson.add(data);
        }
        return resultJson;
    }

    @Override
    public JSONArray savingYoyEnergyData(ScreenRequestParamDTO screenRequestParamDTO) {
        // 判断类型
        JSONArray resultJson = new JSONArray();
        switch (screenRequestParamDTO.getTimeType()) {
            case "day":
                String projectId = screenRequestParamDTO.getProjectId();
                String startTime = screenRequestParamDTO.getStartTime();
                String endTime = screenRequestParamDTO.getEndTime();
                String lastStartTime = DateUtil.yoyDate(startTime + " 00:00:00").substring(0, 10);
                String lastEndTime = DateUtil.yoyDate(endTime+ " 23:59:59").substring(0, 10);
                List<Map<String, Object>> monthData = screenMapper.savingYoyEnergyData(projectId, startTime, endTime, lastStartTime, lastEndTime);
                resultJson.add(monthData);
                break;
            default:
                break;
        }
        return resultJson;
    }

    @Override
    public JSONArray savingEnergy(ScreenRequestParamDTO screenRequestParamDTO) {
        // 判断类型
        JSONArray resultJson = new JSONArray();
        switch (screenRequestParamDTO.getTimeType()) {
            case "day":
                screenRequestParamDTO.setStartTime(screenRequestParamDTO.getStartTime());
                screenRequestParamDTO.setEndTime(screenRequestParamDTO.getEndTime());
                List<Map<String, Object>> monthData = screenMapper.savingEnergy(screenRequestParamDTO);
                resultJson.add(monthData);
                break;
            default:
                break;
        }
        return resultJson;
    }

    @Override
    public JSONArray projectData() {
        JSONArray resultJson = new JSONArray();
        List<Map<String, Object>> projectData = screenMapper.queryProjectData();
        if (projectData.isEmpty()) {
            return resultJson;
        }
        for (Map<String, Object> map : projectData) {
            JSONObject data = new JSONObject();
            float[] ll = new float[2];
            data.put("name", map.get("project_name"));
            if (map.get("longitude") == null || map.get("latitude") == null) {
                continue;
            }
            ll[0] = Float.parseFloat(map.get("longitude").toString());
            ll[1] = Float.parseFloat(map.get("latitude").toString());
            data.put("value", ll);
            data.put("fault", map.get("fault"));
            resultJson.add(data);
        }
        return resultJson;
    }

    @Override
    public JSONArray energyData(ScreenRequestParamDTO screenRequestParamDTO) {
        // 判断类型
        JSONArray resultJson = new JSONArray();
        switch (screenRequestParamDTO.getTimeType()) {
            case "now":
                getNowData(resultJson);
                break;
            case "day":
                screenRequestParamDTO.setStartTime(screenRequestParamDTO.getStartTime());
                screenRequestParamDTO.setEndTime(screenRequestParamDTO.getEndTime());
                getMonthData(resultJson, screenRequestParamDTO);
                break;
            default:
                break;
        }
        return resultJson;
    }

    private void getMonthData(JSONArray resultJson, ScreenRequestParamDTO screenRequestParamDTO) {
        List<Map<String, Object>> monthData = screenMapper.queryMonthData(screenRequestParamDTO);
        resultJson.add(monthData);
    }

    private void getNowData(JSONArray resultJson) {
        List<Map<String, Object>> nowData = screenMapper.queryNowData();
        if (nowData.isEmpty()) {
            return;
        }

        // 根据项目名进行分组
        Map<Object, List<Map<String, Object>>> projectIds = nowData.stream().collect(Collectors.groupingBy(map -> map.get("id")));
        // 根据项目名获取对应的冷量计、电表数据，计算EER
        for (Map.Entry<Object, List<Map<String, Object>>> entry : projectIds.entrySet()) {
            Long proId = (Long) entry.getKey();
            List<Map<String, Object>> projectId = nowData.stream().filter(map -> map.get("id").equals(proId)).collect(Collectors.toList());
            if (projectId.isEmpty()) {
                continue;
            }
            String projectName = (String) projectId.get(0).get("project_name");
            log.info("项目名:{}", projectName);
            String cooling = "未采集";
            String ele = "未采集";
            JSONObject data = new JSONObject();
            data.put("projectName", projectName);
            for (Map<String, Object> map : projectId) {
                // 获取冷量计:2、电表数据:1
                Long mtType = (Long) map.get("mt_type");
                if (null == mtType) {
                    continue;
                }
                if (1 == mtType) {
                    ele = ((BigDecimal) map.get("cur_value")).setScale(2, RoundingMode.HALF_UP).toString();
                }
                if (2 == mtType) {
                    cooling = ((BigDecimal) map.get("cur_value")).setScale(2, RoundingMode.HALF_UP).toString();
                }
            }
            JSONObject dataDetail = new JSONObject();
            dataDetail.clear();

            HashMap<String, Object> map = new HashMap<>();
            map.put("name", projectName);
            map.put("value", ele);
            dataDetail.put("ele", new HashMap<>(map));

            map.clear();
            map.put("name", projectName);
            map.put("value", cooling);
            dataDetail.put("cooling", new HashMap<>(map));

            // 计算EER
            map.clear();
            if ("未采集".equals(cooling) || "未采集".equals(ele)) {
                map.put("name", projectName);
                map.put("value", "未采集");
                dataDetail.put("eer", new HashMap<>(map));
            } else {
                double coolingValue = Double.parseDouble(cooling);
                double eleValue = Double.parseDouble(ele);
                double eer = 0;
                if (eleValue <= 0 || coolingValue <= 0) {
                    eer = 0;
                } else {
                    eer = coolingValue / eleValue;
                }
                map.put("name", projectName);
                map.put("value", (new BigDecimal(eer).setScale(2, RoundingMode.HALF_UP)).toString());
                dataDetail.put("eer", new HashMap<>(map));
            }
            data.put("data", dataDetail);
            resultJson.add(data);
        }
    }
}
