package com.mh.user.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.benmanes.caffeine.cache.Cache;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.mh.common.page.MybatisPageHelper;
import com.mh.common.page.PageRequest;
import com.mh.common.page.PageResult;
import com.mh.common.utils.StringUtils;
import com.mh.user.constants.Constant;
import com.mh.user.entity.ProjectInfoEntity;
import com.mh.user.mapper.ProjectInfoMapper;
import com.mh.user.mapper.SysDictMapper;
import com.mh.user.model.SysDict;
import com.mh.user.service.SysDictService;
import com.mh.user.utils.SecurityUtils;
import com.mh.user.utils.SpringBeanUtil;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description
 * @date 2024-08-23 11:13:38
 */
@Service
public class SysDictServiceImpl implements SysDictService {

    @Autowired
    private SysDictMapper sysDictMapper;

    @Autowired
    private ProjectInfoMapper projectInfoMapper;

    @Autowired
    @Qualifier("caffeineCache")
    private Cache caffeineCache;

    @Override
    public boolean deleteByPrimaryKey(Long id) {
        return sysDictMapper.deleteByPrimaryKey(id);
    }

    @Override
    public List<SysDict> findByLabel(String label) {
        return Collections.emptyList();
    }

    @Override
    public int save(SysDict record) {
        record.setCreateBy(SecurityUtils.getUsername());
        record.setCreateTime(new Date());
        return sysDictMapper.insert(record);
    }

    @Override
    public int delete(SysDict record) {
        return sysDictMapper.deleteById(record);
    }

    @Override
    public int delete(List<SysDict> records) {
        return 0;
    }

    @Override
    public SysDict findById(Long id) {
        return sysDictMapper.selectById(id);
    }

    @Override
    public PageResult findPage(PageRequest pageRequest) {
        String systemId = StringUtils.getColumnFilterValue(pageRequest, "systemId");
        String type = StringUtils.getColumnFilterValue(pageRequest, "type");
        // 设置分页参数
        int pageNum = pageRequest.getPageNum();
        int pageSize = pageRequest.getPageSize();
        if (pageNum != 0) {
            PageHelper.startPage(pageNum, pageSize);
        }
        QueryWrapper<SysDict> sysDictQueryWrapper = new QueryWrapper<>();
        if (!StringUtils.isBlank(type)) {
            sysDictQueryWrapper.eq("type", type);
        }
        sysDictQueryWrapper.ne("parent_id", 0);
        sysDictQueryWrapper.orderByAsc("sort");
        List<SysDict> sysDictList = sysDictMapper.selectList(sysDictQueryWrapper);
        return MybatisPageHelper.getPageResult(pageRequest, new PageInfo<>(sysDictList));
    }

    @Override
    public void updateByPrimaryKey(SysDict entity) {
        entity.setLastUpdateBy(SecurityUtils.getUsername());
        entity.setLastUpdateTime(new Date());
        sysDictMapper.updateById(entity);
    }

    @Override
    public List<SysDict> findByType(String type) {
        return sysDictMapper.selectList(new QueryWrapper<SysDict>().eq("type", type));
    }

    @Override
    public void createDictDataCache() {
        // 删除上次缓存
        caffeineCache.invalidate("sys_dict");
        List<SysDict> sysDictList = sysDictMapper.selectList(
                new QueryWrapper<SysDict>().orderByDesc("create_time"));
        // 把项目列表放到缓存中
        List<ProjectInfoEntity> projectInfoEntities = projectInfoMapper.selectList(
                new QueryWrapper<ProjectInfoEntity>().orderByAsc("create_time")
        );
        // 转换成SysDict,然后插入到sysDictList中
        for (ProjectInfoEntity projectInfoEntity : projectInfoEntities) {
            SysDict sysDict = new SysDict();
            sysDict.setLabel(projectInfoEntity.getProjectName());
            sysDict.setValue(String.valueOf(projectInfoEntity.getId()));
            sysDict.setType("project_name");
            sysDictList.add(sysDict);
        }
        if (caffeineCache.getIfPresent(Constant.SYS_DICT) == null) {
            caffeineCache.put("sys_dict", sysDictList);
        }
    }

    @Override
    public List<SysDict> findCacheByType(String type) {
        List<SysDict> sysDictList = (List<SysDict>) caffeineCache.getIfPresent("sys_dict");
        if (sysDictList == null || sysDictList.size() == 0) {
            createDictDataCache();
            sysDictList = (List<SysDict>) caffeineCache.getIfPresent("sys_dict");
        }
        return sysDictList.stream().filter(item -> item.getType().equals(type)).collect(Collectors.toList());
    }

    @Override
    public List<SysDict> queryTypes() {
        return sysDictMapper.selectList(new QueryWrapper<SysDict>().eq("parent_id", 0).orderByAsc("sort"));
    }
}
