package com.mh.user.strategy;

import com.mh.common.utils.StringUtils;
import com.mh.user.constants.Constant;
import com.mh.user.entity.DataResultEntity;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.MeterManageEntity;
import com.mh.user.service.DataResultService;
import com.mh.user.service.DeviceCodeParamService;
import com.mh.user.service.MeterManageService;
import com.mh.user.service.ProjectInfoService;
import com.mh.user.service.chillers.ChillersService;
import com.mh.user.service.chillers.OrderMessageService;
import com.mh.user.utils.ExchangeStringUtil;
import com.mh.user.utils.SpringBeanUtil;
import com.mh.user.utils.ThreadPoolService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.context.ApplicationContext;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 水表标准协议
 * @date 2024-09-12 14:04:01
 */
@Slf4j
public class EleProtocolStrategy implements ProtocolStrategy {

    // 调用service
    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    ThreadPoolExecutor threadPoolService = ThreadPoolService.getInstance();
    DataResultService dataResultService = context.getBean(DataResultService.class);
    ProjectInfoService projectInfoService = context.getBean(ProjectInfoService.class);

    private static final SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    private static class SingletonHolder {
        private static final EleProtocolStrategy INSTANCE = new EleProtocolStrategy();
    }

    private EleProtocolStrategy() {
        // 防止外部直接实例化
    }

    public static EleProtocolStrategy getInstance() {
        return SingletonHolder.INSTANCE;
    }

    @Override
    public String createOrder(MeterManageEntity meterManageEntity) {
        log.info("电表97/07规约协议：策略创建报文");
        String str = "";
        String deviceAddr = meterManageEntity.getMtCode();
        if (deviceAddr != null && !deviceAddr.isEmpty()) {
            try {
                //0代表前面补充0,12代表长度为12,d代表参数为正数型,基表通讯号
                str = String.format("%012d", Long.parseLong(deviceAddr));
                //转换位置
                str = ExchangeStringUtil.changePosition(str);
                //拼接功能码
                str = "68" + str + "68";
                String strData = "";
                // 判断是什么协议
                switch (meterManageEntity.getProtocolType()) {
                    case 2:
                        // 97协议
                        str += ExchangeStringUtil.addZeroForNum(meterManageEntity.getFuncCode(), 2) + "02";
                        strData = ExchangeStringUtil.addZeroForNum(meterManageEntity.getIdentifyCode(), 4);
                        break;
                    case 3:
                        // 07协议
                        str += ExchangeStringUtil.addZeroForNum(meterManageEntity.getFuncCode(), 2) + "04";
                        strData = ExchangeStringUtil.addZeroForNum(meterManageEntity.getIdentifyCode(), 8);
                        break;
                    default:
                        break;
                }
                //加33操作
                str = str + ExchangeStringUtil.addThree(strData);
                //检验和
                String checkSum = ExchangeStringUtil.makeChecksum(str);
                str = "FEFEFE" + str + checkSum + "16";
            } catch (Exception e) {
                log.error("生成采集电表指令错误==>", e);
                return Constant.FAIL;
            }
        }
        log.info("生成采集电表指令==>表号：{}，指令：{}", meterManageEntity.getMtCode(), str);
        return str.toUpperCase();
    }

    @Override
    public String analysisReceiveData(DeviceCodeParamEntity deviceCodeParamEntity, String receiveData) {
        log.info("电表97/07规约协议：工厂解析报文，接收到的报文<=={}",receiveData);
        String data = "";
        if (receiveData.length() == 34 || receiveData.length() == 36 || receiveData.length() == 40 || receiveData.length() == 44 || receiveData.length() == 50) {
            String str1 = receiveData.substring(0, 8);
            String str2 = receiveData.substring(8);
            receiveData = str1.replace("FE", "") + str2;
            String checkStr = receiveData.substring(0, receiveData.length() - 4); //减去校验码
            String checkNum = ExchangeStringUtil.makeChecksum(checkStr);  //生成校验码
            //返回的校验码与重新生成的校验码进行校验
            if (checkNum.equalsIgnoreCase(receiveData.substring(receiveData.length() - 4, receiveData.length() - 2))) {
                //表号，12位
                String meterId = checkStr.substring(12, 14) + checkStr.substring(10, 12) + checkStr.substring(8, 10)
                        + checkStr.substring(6, 8) + checkStr.substring(4, 6) + checkStr.substring(2, 4);
                meterId = String.format("%012d", Long.parseLong(meterId));
                data = getAnalysisData(receiveData, checkStr, meterId);
            } else {
                log.info("电表报文检验失败: {}", receiveData);
            }
        }
        if (StringUtils.isBlank(data) || Constant.FAIL.equals(data)) {
            return Constant.FAIL;
        }
        data = String.valueOf(Double.valueOf(data)); //00010.76,去除读数前面带0的情况
        // 解析入库
        analysisMeterOrder485(data, deviceCodeParamEntity);
        return data;
    }

    private static String getAnalysisData(String receiveData, String checkStr, String meterId) {
        StringBuilder stringBuilder = new StringBuilder();
        // 累计总读数据：FE FE FE FE 68 69 04 00 70 01 08 68 81 06 43 C3 A5 33 33 33 81 16
        // 瞬时有功功率：FE FE FE FE 68 69 04 00 70 01 08 68 81 05 63 E9 33 33 33 21 16
        //            FE FE FE FE 68 98 01 50 04 00 00 68 C1 01 35 B4 16
        // 判断属于哪个数据标识
        String resultData = Constant.FAIL;
        if (receiveData.length() > 24) {
            String identifyCode = ExchangeStringUtil.changePosition(ExchangeStringUtil.cutThree(receiveData.substring(20, 24)));
            switch (identifyCode) {
                case "9010":
                    // 累计总读数据
                    for (int i = 0; i < 4; i++) {
                        String data1 = checkStr.substring(32 - 2 * (i + 1), 32 - 2 * i);
                        stringBuilder.append(data1);
                    }
                    resultData = stringBuilder.toString();
                    resultData = ExchangeStringUtil.cutThree(resultData);
                    // 0 代表前面补充0,4 代表长度为4,d 代表参数为正数型
                    resultData = String.format("%08d", Long.parseLong(resultData));
                    resultData = resultData.substring(0, 6) + "." + resultData.substring(6, 8);
                    log.info("电表表号：{},电表累计总量读数：{}", meterId, resultData);
                    break;
                case "B630":
                    // 瞬时有功功率
                    for (int i = 0; i < 3; i++) {
                        String data1 = checkStr.substring(30 - 2 * (i + 1), 30 - 2 * i);
                        stringBuilder.append(data1);
                    }
                    resultData = stringBuilder.toString();
                    resultData = ExchangeStringUtil.cutThree(resultData);
                    // 0 代表前面补充0,4 代表长度为4,d 代表参数为正数型
                    resultData = String.format("%06d", Long.parseLong(resultData));
                    resultData = resultData.substring(0, 2) + "." + resultData.substring(2, 6);
                    log.info("电表表号：{},电表瞬时有功功率读数：{}", meterId, resultData);
                    break;
            }
        }
        return resultData;
    }

    /**
     * 解析电表返回的数据
     *
     * @param dataStr
     */
    public void analysisMeterOrder485(final String dataStr, final DeviceCodeParamEntity deviceCodeParam) {
        threadPoolService.execute(() -> {
            Date date = new Date();
            String dateStr = sdf1.format(date);
            try {
                DataResultEntity dataResultEntity = new DataResultEntity();
                dataResultEntity.setDeviceAddr(deviceCodeParam.getDeviceAddr());    //通讯编号
                dataResultEntity.setDeviceType(deviceCodeParam.getDeviceType());                              //类型
                dataResultEntity.setProjectId(deviceCodeParam.getProjectId());      //所属项目
                dataResultEntity.setCurValue(new BigDecimal(dataStr));              //当前读数
                dataResultEntity.setCurDate(sdf1.parse(dateStr));                   //当前日期
                dataResultEntity.setGrade(deviceCodeParam.getGrade());
                dataResultEntity.setRegisterAddr(deviceCodeParam.getRegisterAddr());
                dataResultEntity.setRegisterName(deviceCodeParam.getRegisterName());
                String projectName = projectInfoService.selectName(deviceCodeParam.getProjectId());
                log.info("电表==>{},读数==>{},项目名称==>{}", deviceCodeParam.getDeviceAddr(), dataStr, projectName);
                dataResultService.saveDataResult(dataResultEntity);
                log.info("电表保存数据成功!项目名称:{}", projectName);
            } catch (Exception e) {
                log.error("保存电表数据失败!", e);
            }
        });
    }

}
